/*
 * Author: Radovan Cerveny
 */

#include "provisioner/MeasurementProvisionerConfigurationXml.hpp"
#include "provisioner/MeasurementProvisioner.hpp"
#include "factory/XmlDataFactory.hpp"
#include <tclap/CmdLine.h>
#include <measure>

int main ( int argc, char * * argv ) {

	try {
		TCLAP::CmdLine cmd ( "Measurement provisioner and processor binary", ' ', "0.01" );

		TCLAP::UnlabeledValueArg < std::string > input ( "input", "provisioning file", false, "-", "file" );
		cmd.add ( input );

		cmd.parse ( argc, argv );

		std::deque < sax::Token > tokens;

		if ( input.getValue ( ) == "-" )
			sax::SaxParseInterface::parseStdin ( tokens );
		else
			sax::SaxParseInterface::parseFile ( input.getValue ( ), tokens );

		auto configuration = provisioner::MeasurementProvisionerConfigurationXml::parse ( tokens );
		auto results = provisioner::MeasurementProvisioner::runConfiguration ( configuration );

		std::cout << results << std::endl;

		return 0;

	} catch ( const exception::AlibException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const exception::CommonException & exception ) {
		std::cerr << "CommonException caught: " << exception.getCause ( ) << std::endl;
		return 4;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
