/*
 * Author: Radovan Cerveny
 */

#include "MeasurementProvisionerResultsXml.hpp"
#include <measure>

using namespace std;
using namespace sax;

namespace provisioner {

deque < Token > MeasurementProvisionerResultsXml::compose ( const MeasurementProvisionerResults & mpr ) {
	deque < Token > tokens;

	tokens.emplace_back ( MEASUREMENT_PROVISIONER_RESULTS_TAG, Token::TokenType::START_ELEMENT );

	for ( const MPRResults & mprr : mpr.results )
		composeMeasurementProvisionerResult ( tokens, mprr );

	tokens.emplace_back ( MEASUREMENT_PROVISIONER_RESULTS_TAG, Token::TokenType::END_ELEMENT );
	return tokens;
}

void MeasurementProvisionerResultsXml::composeMeasurementProvisionerResult ( deque < Token > & tokens, const MPRResults & mprr ) {
	tokens.emplace_back ( MEASUREMENT_PROVISIONER_RESULT_TAG, Token::TokenType::START_ELEMENT );

	composeInputs ( tokens, mprr );
	composePipelines ( tokens, mprr );

	tokens.emplace_back ( MEASUREMENT_PROVISIONER_RESULT_TAG, Token::TokenType::END_ELEMENT );
}

void MeasurementProvisionerResultsXml::composeInputs ( deque < Token > & tokens, const MPRResults & mprr ) {
	tokens.emplace_back ( INPUTS_TAG, Token::TokenType::START_ELEMENT );

	for ( const std::pair < int, std::string > & input : mprr.inputs ) {
		tokens.emplace_back ( INPUT_TAG, Token::TokenType::START_ELEMENT );
		tokens.emplace_back ( INPUT_ATTR_ID_NAME, Token::TokenType::START_ATTRIBUTE );
		tokens.emplace_back ( to_string ( input.first ), Token::TokenType::CHARACTER );
		tokens.emplace_back ( INPUT_ATTR_ID_NAME, Token::TokenType::END_ATTRIBUTE );
		tokens.emplace_back ( input.second, Token::TokenType::CHARACTER );
		tokens.emplace_back ( INPUT_TAG, Token::TokenType::END_ELEMENT );
	}

	tokens.emplace_back ( INPUTS_TAG, Token::TokenType::END_ELEMENT );
}

void MeasurementProvisionerResultsXml::composePipelines ( deque < Token > & tokens, const MPRResults & mprr ) {
	tokens.emplace_back ( PIPELINES_TAG, Token::TokenType::START_ELEMENT );

	for ( const MPRPipelineResults & mprpr : mprr.pipelineResults )
		composePipeline ( tokens, mprpr );

	tokens.emplace_back ( PIPELINES_TAG, Token::TokenType::END_ELEMENT );
}

void MeasurementProvisionerResultsXml::composePipeline ( deque < Token > & tokens, const MPRPipelineResults & mprpr ) {
	tokens.emplace_back ( PIPELINE_TAG, Token::TokenType::START_ELEMENT );

	composePipelineStatus ( tokens, mprpr );
	composeCommands ( tokens, mprpr );

	tokens.emplace_back ( PIPELINE_TAG, Token::TokenType::END_ELEMENT );
}

void MeasurementProvisionerResultsXml::composePipelineStatus ( deque < Token > & tokens, const MPRPipelineResults & mprpr ) {

	tokens.emplace_back ( PIPELINE_STATUS_TAG, Token::TokenType::START_ELEMENT );
	tokens.emplace_back ( EXIT_CODE_TAG, Token::TokenType::START_ELEMENT );
	tokens.emplace_back ( to_string ( mprpr.pipelineStatus.exitCode ), Token::TokenType::CHARACTER );
	tokens.emplace_back ( EXIT_CODE_TAG, Token::TokenType::END_ELEMENT );
	tokens.emplace_back ( ERROR_ORIGIN_TAG, Token::TokenType::START_ELEMENT );
	tokens.emplace_back ( mprpr.pipelineStatus.errorOrigin, Token::TokenType::CHARACTER );
	tokens.emplace_back ( ERROR_ORIGIN_TAG, Token::TokenType::END_ELEMENT );
	tokens.emplace_back ( ERROR_VALUE_TAG, Token::TokenType::START_ELEMENT );
	tokens.emplace_back ( mprpr.pipelineStatus.errorValue, Token::TokenType::CHARACTER );
	tokens.emplace_back ( ERROR_VALUE_TAG, Token::TokenType::END_ELEMENT );

	tokens.emplace_back ( PIPELINE_STATUS_TAG, Token::TokenType::END_ELEMENT );
}

void MeasurementProvisionerResultsXml::composeCommands ( deque < Token > & tokens, const MPRPipelineResults & mprpr ) {
	tokens.emplace_back ( COMMANDS_TAG, Token::TokenType::START_ELEMENT );

	for ( const MPRPipelineCommandResults & mprpcr : mprpr.commandResults )
		composeCommand ( tokens, mprpcr );

	tokens.emplace_back ( COMMANDS_TAG, Token::TokenType::END_ELEMENT );
}

void MeasurementProvisionerResultsXml::composeCommand ( deque < Token > & tokens, const MPRPipelineCommandResults & mprpcr ) {
	tokens.emplace_back ( COMMAND_TAG, Token::TokenType::START_ELEMENT );
	tokens.emplace_back ( COMMAND_NAME_TAG, Token::TokenType::START_ELEMENT );
	tokens.emplace_back ( mprpcr.command, Token::TokenType::CHARACTER );
	tokens.emplace_back ( COMMAND_NAME_TAG, Token::TokenType::END_ELEMENT );

	deque < Token > measurementResultsTokens = measurements::MeasurementXmlFactory::composeMeasurementResults ( mprpcr.measurementResults );

	for ( Token & tok : measurementResultsTokens )
		tokens.emplace_back ( std::move ( tok ) );

	tokens.emplace_back ( COMMAND_TAG, Token::TokenType::END_ELEMENT );
}

string MeasurementProvisionerResultsXml::MEASUREMENT_PROVISIONER_RESULTS_TAG = "MeasurementProvisionerResults";

string MeasurementProvisionerResultsXml::MEASUREMENT_PROVISIONER_RESULT_TAG = "MeasurementProvisionerResult";

string MeasurementProvisionerResultsXml::INPUTS_TAG = "Inputs";

string MeasurementProvisionerResultsXml::INPUT_TAG = "Input";
string MeasurementProvisionerResultsXml::INPUT_ATTR_ID_NAME = "id";

string MeasurementProvisionerResultsXml::PIPELINES_TAG = "Pipelines";
string MeasurementProvisionerResultsXml::PIPELINE_TAG  = "Pipeline";

string MeasurementProvisionerResultsXml::PIPELINE_STATUS_TAG = "PipelineStatus";
string MeasurementProvisionerResultsXml::EXIT_CODE_TAG = "ExitCode";
string MeasurementProvisionerResultsXml::ERROR_ORIGIN_TAG = "ErrorOrigin";
string MeasurementProvisionerResultsXml::ERROR_VALUE_TAG	= "ErrorValue";

string MeasurementProvisionerResultsXml::COMMANDS_TAG = "Commands";
string MeasurementProvisionerResultsXml::COMMAND_TAG  = "Command";

string MeasurementProvisionerResultsXml::COMMAND_NAME_TAG = "Name";

string MeasurementProvisionerResultsXml::MEASUREMENT_RESULTS_TAG = "MeasurementResults";
}
