/*
 * Author: Radovan Cerveny
 */

#include "MeasurementProvisionerUtils.hpp"
#include "exception/CommonException.h"
#include <wordexp.h>
#include <unistd.h>
#include <iostream>

namespace provisioner {

std::list < std::string > MPUtils::shellExpand ( const std::string & str ) {
	std::list < std::string > results;

	wordexp_t wexp;

	if ( wordexp ( str.c_str ( ), & wexp, 0 ) != 0 )
		throw::exception::CommonException ( "MPUtils: wordexp failed" );

	for ( unsigned i = 0; i < wexp.we_wordc; ++i )
		results.emplace_back ( wexp.we_wordv[i] );

	wordfree ( & wexp );

	return results;
}

std::string MPUtils::generateTmpfileFromCommand ( const std::string & command ) {
	char temppath[] = "/tmp/genXXXXXX";

	int tempfd = mkstemp ( temppath );

	if ( tempfd == -1 )
		throw::exception::CommonException ( "MPUtils: mkstemp failed" );

	int stdoutfd = dup ( 1 );
	close ( 1 );
	dup ( tempfd );


	int status = system ( command.c_str ( ) );

	close ( 1 );
	close ( tempfd );
	dup ( stdoutfd );

	if ( status != 0 ) {
		unlink ( temppath );
		throw::exception::CommonException ( "MPUtils: system call failed" );
	}

	return std::string ( temppath );
}

}
